# __Start Homework 2 ####
#............................
# Births 2012 Analysis for EPID 799C: PNC decrease PT birth? ####
# Mike Dolan Fliss, August 2017
# Aim: Does early prenatal care (<month 5) decrease the risk of preterm (<week 37) birth?
# Project Notes:
# Birth Certificate Revisions: 
# https://www.cdc.gov/nchs/nvss/vital_certificate_revisions.htm (HW1.Q1d, clickable link)
#............................

#............................
# Load libraries & recoding script
#............................
# setwd("B:/20Spring_EPID701_RforEpi/") # Mike's home dir. Cheating by mapping a drive w subst on PC
# setwd("C:/Users/fliss/Dropbox/Education/Classes/20Spring_EPID701_RforEpi") # Mike's home dir.
setwd("D:/User/Dropbox (Personal)/Education/Classes/20Spring_EPID701_RforEpi") # Mike's work dir.
source("Homework/Births analysis 2020_HW1_answers.R") 
# load("births_cleaned_hw1.rda")
# See Jenny Bryan naming conventions... in practice I do something like this:
# source("00_libraries_and_constants.R") 
# source("01_load_and_recode.R") # uncomment to rerun

#............................

# ......................................
# Eligibility criteria & core recoding  (HW2 part 1) ####
# ......................................
# Create many inclusion variables, all of which have to be =1 to be included.
births$weeknum = births$dob_d %>% week #thanks, lubridate! (HW2.1.Q1)
# a. Has gestation data (weeks), and at least 20 weeks gestation pre-birth (weeks >= 20)
births$wksgest[births$wksgest == 99] = NA # < Ishrat, good catch! Won't impact final data, but good to be explicit.
births$incl_hasgest = !is.na(births$wksgest) %>% as.numeric #(HW2.1.Q2a)
births$incl_enoughgest = as.numeric(births$wksgest >= 20) #(HW2.1.Q2b)
# b. Less than 20 weeks gestation before Jan 1 of year (LMP > Aug 20), or weeks-weeknum>=19, w/ weeknum=1 for Jan 1-7
births$incl_lateenough = as.numeric(births$wksgest - births$weeknum <= 19) #(HW2.1.Q2c)
# c. Start date of gestation was 45 (max gest in'11) w prior to Jan 1 of next year, so all births are observed in year
births$incl_earlyenough = as.numeric(births$weeknum - births$wksgest <=7) #(HW2.1.Q2d)
# d. singletons only
births$incl_singleton = as.numeric(births$plur == 1) #(HW2.Q2e)
# births$incl_hasnumdata = as.numeric(births$plur != 9) # not bothering to recode is ok. Similar / redundant to above, though I often like to be explicit about NAs. -Mike

# e. no congential anomalies - Let's use apply for efficiency

# Warm up w/ apply - when I was learning purrr/apply I'd often build a trivial example to test on. -Mike
m1 = matrix(1:20, nrow=4);m1
apply(m1, 1, sum)
apply(m1, 2, sum)
all(list("N", "N", "Y") == "N") # Test an idea.

congen_anom = c("anen","mnsb","cchd", "cdh", "omph","gast", "limb", "cl","cp","dowt","cdit", "hypo") #(HW2.1.Q3a)
births[,congen_anom] %>% head
births[,congen_anom] %>% map(table, useNA = "always")
test_df = tibble(a = rep("N", 3), b = c("N", "U", "Y")); test_df 
test_df == "N"
births$incl_noanomalies = as.numeric(apply(births[,congen_anom]=="N", FUN=all, 1)) #All not present #(HW2.1.Q3b)

# .write a function! Give 1 row of df / list, is all "N"?
check_for_anomalies = function(...){ all(c(...) == "N") } 
pmap_lgl(test_df, check_for_anomalies) # test on df

births$incl_noanomalies2 =  #All not present #(HW2.1.Q3c)
  births[,congen_anom] %>% pmap_lgl(check_for_anomalies) %>% 
  as.numeric
identical(births$incl_noanomalies, births$incl_noanomalies2)

# Optional C. Create congen_anom from the meta data file. 
meta = readxl::read_excel("R for epi 2020 data pack/resources/2011-present Birth Dataset Description_Stat.xls", skip = 2)
congen_anom2 = meta %>% filter(str_detect(Variable, "Anomal")) %>% pull(`SAS Name`) %>% # big chain!
  paste0(collapse = "") %>% str_trim %>% str_split("[\n]+", simplify = T) %>% as.vector %>% tolower 
congen_anom2

# Below is the example grepl (HW2.1.Q4b)
grepl("a", c("banana", "peach", "ornge"))
str_detect( c("banana", "peach", "ornge"), "a") 
c("banana", "peach", "ornge") %>% str_detect("a") # especially when piped, I fidn this more readable. "...then detect an 'a'"
eligibility_drops = nrow(births) - apply(births[, grepl("incl_", names(births))], FUN=sum, MARGIN = 2, na.rm=T) #(HW2.1.Q4c optional)
eligibility_drops = nrow(births) - apply(births[, names(births) %>% str_detect("incl_")], FUN=sum, MARGIN = 2, na.rm=T) #(HW2.1.Q4c optional)
eligibility_drops #(HW2.1.Q4c optional)

births$include_allpass = apply(births[, grepl("incl_", names(births))], FUN=all, MARGIN = 1) #(HW2.1.Q4d indicator for the inclusion criteria yes/no)
# contains! births$include_allpass = apply(births[, grepl("incl_", names(births))], FUN=all, MARGIN = 1) #(HW2.1.Q4d indicator for the inclusion criteria yes/no)

names(births)[str_detect(names(births), "w")]

old_births <- births #(HW2.1.Q4a) # bumped down to do the optional challenge

births = births[births$include_allpass, ] #(HW2.1.Q4e now apply the inclusion criteria to subset your births data set)
# to see what warnings are being thrown you can type warnings() -- below:
warnings() # Just letting me know I'm casting those 1s as TRUEs. That's ok.

dim(old_births) #(HW2.1.Q4f)
dim(births) #(HW2.1.Q4f)

#(HW2.1.Q5 Optional Challenge)
# Results of exclusion
cat("Leaving eligibility checks with n =", prettyNum(nrow(births), big.mark = ","),
    "births of original", prettyNum(nrow(old_births), big.mark = ",")) #(HW2.1.Q4d)
# ^ In practice I might actually save these sorts of sentences in a list or data.frame 
# (e.g. analysis_notes$eligibility_note) and save it as I go.

#5A
names(births)
vars_of_interest = c("pnc5_f", "preterm_f", "smoker_f", "wksgest", "sex", "meduc", "mage", 
                     "raceeth_f", "weeknum", "include_allpass", "cores", "kotel")



# Save a smaller version for future loading - 5B
vars_to_save = c(vars_of_interest)
births_sm = births[, vars_to_save] %>% as_tibble()
save("births_sm", file = "births_sm.rdata")
save("births_sm", file = "births_02.rdata")

#Optional 
tableone::CreateTableOne(data=births[, vars_of_interest]) #(HW2.1.Q6)
tableone::CreateTableOne(data=old_births[, vars_of_interest], strata="include_allpass")

#Optional 
Hmisc::describe(births)

# ......................................
# Finishes with 62,370 of 122,513 ####
# ......................................
table(births$wksgest)
